Switch FP creation over to GrRecordingContext
This CL does make use of a new GrRecordingContextPriv backdoor to break CL deadlocks. This occurs when this CL tries to create GrContext-dependent objects outside its scope.
Change-Id: I925030c818f00559d4c953ae07af53667b44aab9
Reviewed-on: https://skia-review.googlesource.com/c/192032
Commit-Queue: Robert Phillips <robertphillips@google.com>
Reviewed-by: Brian Salomon <bsalomon@google.com>
diff --git a/src/core/SkImageGenerator.cpp b/src/core/SkImageGenerator.cpp
index 8b17911..611f7a4 100644
--- a/src/core/SkImageGenerator.cpp
+++ b/src/core/SkImageGenerator.cpp
@@ -61,7 +61,8 @@
#if SK_SUPPORT_GPU
#include "GrTextureProxy.h"
-sk_sp<GrTextureProxy> SkImageGenerator::generateTexture(GrContext* ctx, const SkImageInfo& info,
+sk_sp<GrTextureProxy> SkImageGenerator::generateTexture(GrRecordingContext* ctx,
+ const SkImageInfo& info,
const SkIPoint& origin,
bool willNeedMipMaps) {
SkIRect srcRect = SkIRect::MakeXYWH(origin.x(), origin.y(), info.width(), info.height());
@@ -71,7 +72,8 @@
return this->onGenerateTexture(ctx, info, origin, willNeedMipMaps);
}
-sk_sp<GrTextureProxy> SkImageGenerator::onGenerateTexture(GrContext*, const SkImageInfo&,
+sk_sp<GrTextureProxy> SkImageGenerator::onGenerateTexture(GrRecordingContext*,
+ const SkImageInfo&,
const SkIPoint&,
bool willNeedMipMaps) {
return nullptr;
diff --git a/src/core/SkPictureImageGenerator.cpp b/src/core/SkPictureImageGenerator.cpp
index 1b10352..ed3cbfa 100644
--- a/src/core/SkPictureImageGenerator.cpp
+++ b/src/core/SkPictureImageGenerator.cpp
@@ -26,8 +26,8 @@
#if SK_SUPPORT_GPU
TexGenType onCanGenerateTexture() const override { return TexGenType::kExpensive; }
- sk_sp<GrTextureProxy> onGenerateTexture(GrContext*, const SkImageInfo&, const SkIPoint&,
- bool willNeedMipMaps) override;
+ sk_sp<GrTextureProxy> onGenerateTexture(GrRecordingContext*, const SkImageInfo&,
+ const SkIPoint&, bool willNeedMipMaps) override;
#endif
private:
@@ -92,12 +92,19 @@
///////////////////////////////////////////////////////////////////////////////////////////////////
#if SK_SUPPORT_GPU
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
+
sk_sp<GrTextureProxy> SkPictureImageGenerator::onGenerateTexture(
- GrContext* ctx, const SkImageInfo& info, const SkIPoint& origin, bool willNeedMipMaps) {
+ GrRecordingContext* ctx, const SkImageInfo& info,
+ const SkIPoint& origin, bool willNeedMipMaps) {
SkASSERT(ctx);
SkSurfaceProps props(0, kUnknown_SkPixelGeometry);
- sk_sp<SkSurface> surface(SkSurface::MakeRenderTarget(ctx, SkBudgeted::kYes, info, 0,
+
+ // CONTEXT TODO: remove this use of 'backdoor' to create an SkSkSurface
+ sk_sp<SkSurface> surface(SkSurface::MakeRenderTarget(ctx->priv().backdoor(),
+ SkBudgeted::kYes, info, 0,
kTopLeft_GrSurfaceOrigin, &props,
willNeedMipMaps));
if (!surface) {
diff --git a/src/gpu/GrAHardwareBufferImageGenerator.cpp b/src/gpu/GrAHardwareBufferImageGenerator.cpp
index 9b75c3e..9220a19 100644
--- a/src/gpu/GrAHardwareBufferImageGenerator.cpp
+++ b/src/gpu/GrAHardwareBufferImageGenerator.cpp
@@ -21,6 +21,8 @@
#include "GrContext.h"
#include "GrContextPriv.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrResourceCache.h"
#include "GrResourceProvider.h"
#include "GrResourceProviderPriv.h"
@@ -78,12 +80,17 @@
///////////////////////////////////////////////////////////////////////////////////////////////////
-sk_sp<GrTextureProxy> GrAHardwareBufferImageGenerator::makeProxy(GrContext* context) {
- if (context->abandoned()) {
+sk_sp<GrTextureProxy> GrAHardwareBufferImageGenerator::makeProxy(GrRecordingContext* context) {
+ if (context->priv().abandoned()) {
return nullptr;
}
- GrBackendFormat backendFormat = GrAHardwareBufferUtils::GetBackendFormat(context,
+ auto direct = context->priv().asDirectContext();
+ if (!direct) {
+ return nullptr;
+ }
+
+ GrBackendFormat backendFormat = GrAHardwareBufferUtils::GetBackendFormat(direct,
fHardwareBuffer,
fBufferFormat,
false);
@@ -142,13 +149,13 @@
};
sk_sp<GrTextureProxy> texProxy = proxyProvider->createLazyProxy(
- [context, buffer = AutoAHBRelease(hardwareBuffer), width, height, pixelConfig,
+ [direct, buffer = AutoAHBRelease(hardwareBuffer), width, height, pixelConfig,
isProtectedContent, backendFormat](GrResourceProvider* resourceProvider) {
GrAHardwareBufferUtils::DeleteImageProc deleteImageProc = nullptr;
GrAHardwareBufferUtils::DeleteImageCtx deleteImageCtx = nullptr;
GrBackendTexture backendTex =
- GrAHardwareBufferUtils::MakeBackendTexture(context, buffer.get(),
+ GrAHardwareBufferUtils::MakeBackendTexture(direct, buffer.get(),
width, height,
&deleteImageProc,
&deleteImageCtx,
@@ -186,7 +193,8 @@
}
sk_sp<GrTextureProxy> GrAHardwareBufferImageGenerator::onGenerateTexture(
- GrContext* context, const SkImageInfo& info, const SkIPoint& origin, bool willNeedMipMaps) {
+ GrRecordingContext* context, const SkImageInfo& info,
+ const SkIPoint& origin, bool willNeedMipMaps) {
sk_sp<GrTextureProxy> texProxy = this->makeProxy(context);
if (!texProxy) {
return nullptr;
diff --git a/src/gpu/GrAHardwareBufferImageGenerator.h b/src/gpu/GrAHardwareBufferImageGenerator.h
index a230b04..95b2fc2 100644
--- a/src/gpu/GrAHardwareBufferImageGenerator.h
+++ b/src/gpu/GrAHardwareBufferImageGenerator.h
@@ -42,14 +42,14 @@
bool onIsValid(GrContext*) const override;
TexGenType onCanGenerateTexture() const override { return TexGenType::kCheap; }
- sk_sp<GrTextureProxy> onGenerateTexture(GrContext*, const SkImageInfo&, const SkIPoint&,
- bool willNeedMipMaps) override;
+ sk_sp<GrTextureProxy> onGenerateTexture(GrRecordingContext*, const SkImageInfo&,
+ const SkIPoint&, bool willNeedMipMaps) override;
private:
GrAHardwareBufferImageGenerator(const SkImageInfo&, AHardwareBuffer*, SkAlphaType,
bool isProtectedContent, uint32_t bufferFormat,
GrSurfaceOrigin surfaceOrigin);
- sk_sp<GrTextureProxy> makeProxy(GrContext* context);
+ sk_sp<GrTextureProxy> makeProxy(GrRecordingContext* context);
void releaseTextureRef();
diff --git a/src/gpu/GrBackendTextureImageGenerator.cpp b/src/gpu/GrBackendTextureImageGenerator.cpp
index c47efd6..0c41cc1 100644
--- a/src/gpu/GrBackendTextureImageGenerator.cpp
+++ b/src/gpu/GrBackendTextureImageGenerator.cpp
@@ -10,6 +10,8 @@
#include "GrContextPriv.h"
#include "GrGpu.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrRenderTargetContext.h"
#include "GrResourceCache.h"
#include "GrResourceProvider.h"
@@ -90,7 +92,8 @@
}
sk_sp<GrTextureProxy> GrBackendTextureImageGenerator::onGenerateTexture(
- GrContext* context, const SkImageInfo& info, const SkIPoint& origin, bool willNeedMipMaps) {
+ GrRecordingContext* context, const SkImageInfo& info,
+ const SkIPoint& origin, bool willNeedMipMaps) {
SkASSERT(context);
if (context->backend() != fBackendTexture.backend()) {
diff --git a/src/gpu/GrBackendTextureImageGenerator.h b/src/gpu/GrBackendTextureImageGenerator.h
index 9d76e02..74dd3ea 100644
--- a/src/gpu/GrBackendTextureImageGenerator.h
+++ b/src/gpu/GrBackendTextureImageGenerator.h
@@ -40,8 +40,8 @@
bool onIsValid(GrContext*) const override { return true; }
TexGenType onCanGenerateTexture() const override { return TexGenType::kCheap; }
- sk_sp<GrTextureProxy> onGenerateTexture(GrContext*, const SkImageInfo&, const SkIPoint&,
- bool willNeedMipMaps) override;
+ sk_sp<GrTextureProxy> onGenerateTexture(GrRecordingContext*, const SkImageInfo&,
+ const SkIPoint&, bool willNeedMipMaps) override;
private:
GrBackendTextureImageGenerator(const SkImageInfo& info, GrTexture*, GrSurfaceOrigin,
diff --git a/src/gpu/GrBitmapTextureMaker.cpp b/src/gpu/GrBitmapTextureMaker.cpp
index 89a3702..adc6347 100644
--- a/src/gpu/GrBitmapTextureMaker.cpp
+++ b/src/gpu/GrBitmapTextureMaker.cpp
@@ -7,10 +7,10 @@
#include "GrBitmapTextureMaker.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrGpuResourcePriv.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrSurfaceContext.h"
#include "SkBitmap.h"
#include "SkGr.h"
@@ -19,7 +19,7 @@
static bool bmp_is_alpha_only(const SkBitmap& bm) { return kAlpha_8_SkColorType == bm.colorType(); }
-GrBitmapTextureMaker::GrBitmapTextureMaker(GrContext* context, const SkBitmap& bitmap)
+GrBitmapTextureMaker::GrBitmapTextureMaker(GrRecordingContext* context, const SkBitmap& bitmap)
: INHERITED(context, bitmap.width(), bitmap.height(), bmp_is_alpha_only(bitmap))
, fBitmap(bitmap) {
if (!bitmap.isVolatile()) {
diff --git a/src/gpu/GrBitmapTextureMaker.h b/src/gpu/GrBitmapTextureMaker.h
index 0602e5a..886014b 100644
--- a/src/gpu/GrBitmapTextureMaker.h
+++ b/src/gpu/GrBitmapTextureMaker.h
@@ -16,7 +16,7 @@
subset of the pixelref specified by the bitmap. */
class GrBitmapTextureMaker : public GrTextureMaker {
public:
- GrBitmapTextureMaker(GrContext* context, const SkBitmap& bitmap);
+ GrBitmapTextureMaker(GrRecordingContext* context, const SkBitmap& bitmap);
protected:
sk_sp<GrTextureProxy> refOriginalTextureProxy(bool willBeMipped,
diff --git a/src/gpu/GrFPArgs.h b/src/gpu/GrFPArgs.h
index cbc0420..94b26e6 100644
--- a/src/gpu/GrFPArgs.h
+++ b/src/gpu/GrFPArgs.h
@@ -12,10 +12,10 @@
#include "SkMatrix.h"
class GrColorSpaceInfo;
-class GrContext;
+class GrRecordingContext;
struct GrFPArgs {
- GrFPArgs(GrContext* context,
+ GrFPArgs(GrRecordingContext* context,
const SkMatrix* viewMatrix,
SkFilterQuality filterQuality,
const GrColorSpaceInfo* dstColorSpaceInfo)
@@ -30,7 +30,7 @@
class WithPreLocalMatrix;
class WithPostLocalMatrix;
- GrContext* fContext;
+ GrRecordingContext* fContext;
const SkMatrix* fViewMatrix;
// We track both pre and post local matrix adjustments. For a given FP:
diff --git a/src/gpu/GrImageTextureMaker.cpp b/src/gpu/GrImageTextureMaker.cpp
index 5e821f5..52b06af 100644
--- a/src/gpu/GrImageTextureMaker.cpp
+++ b/src/gpu/GrImageTextureMaker.cpp
@@ -12,7 +12,7 @@
#include "SkImage_Lazy.h"
#include "effects/GrYUVtoRGBEffect.h"
-GrImageTextureMaker::GrImageTextureMaker(GrContext* context, const SkImage* client,
+GrImageTextureMaker::GrImageTextureMaker(GrRecordingContext* context, const SkImage* client,
SkImage::CachingHint chint)
: INHERITED(context, client->width(), client->height(), client->isAlphaOnly())
, fImage(static_cast<const SkImage_Lazy*>(client))
diff --git a/src/gpu/GrImageTextureMaker.h b/src/gpu/GrImageTextureMaker.h
index 544705f..76f539d 100644
--- a/src/gpu/GrImageTextureMaker.h
+++ b/src/gpu/GrImageTextureMaker.h
@@ -18,7 +18,8 @@
is kAllow the image's ID is used for the cache key. */
class GrImageTextureMaker : public GrTextureMaker {
public:
- GrImageTextureMaker(GrContext* context, const SkImage* client, SkImage::CachingHint chint);
+ GrImageTextureMaker(GrRecordingContext* context, const SkImage* client,
+ SkImage::CachingHint chint);
protected:
// TODO: consider overriding this, for the case where the underlying generator might be
diff --git a/src/gpu/GrRecordingContext.cpp b/src/gpu/GrRecordingContext.cpp
index 20b857e..47adb97 100644
--- a/src/gpu/GrRecordingContext.cpp
+++ b/src/gpu/GrRecordingContext.cpp
@@ -280,3 +280,7 @@
mipMapped, origin, surfaceProps,
budgeted);
}
+
+GrContext* GrRecordingContextPriv::backdoor() {
+ return (GrContext*) fContext;
+}
diff --git a/src/gpu/GrRecordingContextPriv.h b/src/gpu/GrRecordingContextPriv.h
index b00d82f..b130bee 100644
--- a/src/gpu/GrRecordingContextPriv.h
+++ b/src/gpu/GrRecordingContextPriv.h
@@ -110,6 +110,10 @@
GrAuditTrail* auditTrail() { return fContext->auditTrail(); }
+ // CONTEXT TODO: remove this backdoor
+ // In order to make progress we temporarily need a way to break CL impasses.
+ GrContext* backdoor();
+
private:
explicit GrRecordingContextPriv(GrRecordingContext* context) : fContext(context) {}
GrRecordingContextPriv(const GrRecordingContextPriv&); // unimpl
diff --git a/src/gpu/GrTextureAdjuster.cpp b/src/gpu/GrTextureAdjuster.cpp
index 8295998..751d646 100644
--- a/src/gpu/GrTextureAdjuster.cpp
+++ b/src/gpu/GrTextureAdjuster.cpp
@@ -7,13 +7,13 @@
#include "GrTextureAdjuster.h"
#include "GrColorSpaceXform.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrGpu.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "SkGr.h"
-GrTextureAdjuster::GrTextureAdjuster(GrContext* context, sk_sp<GrTextureProxy> original,
+GrTextureAdjuster::GrTextureAdjuster(GrRecordingContext* context, sk_sp<GrTextureProxy> original,
SkAlphaType alphaType,
uint32_t uniqueID,
SkColorSpace* cs)
@@ -37,7 +37,7 @@
sk_sp<GrTextureProxy> GrTextureAdjuster::refTextureProxyCopy(const CopyParams& copyParams,
bool willBeMipped) {
- GrProxyProvider* proxyProvider = fContext->priv().proxyProvider();
+ GrProxyProvider* proxyProvider = this->context()->priv().proxyProvider();
GrUniqueKey key;
this->makeCopyKey(copyParams, &key);
@@ -52,7 +52,8 @@
sk_sp<GrTextureProxy> proxy = this->originalProxyRef();
- sk_sp<GrTextureProxy> copy = CopyOnGpu(fContext, std::move(proxy), copyParams, willBeMipped);
+ sk_sp<GrTextureProxy> copy = CopyOnGpu(this->context(), std::move(proxy),
+ copyParams, willBeMipped);
if (copy) {
if (key.isValid()) {
SkASSERT(copy->origin() == this->originalProxy()->origin());
@@ -79,20 +80,20 @@
sk_sp<GrTextureProxy> proxy = this->originalProxyRef();
CopyParams copyParams;
- if (!fContext) {
+ if (this->context()->priv().abandoned()) {
// The texture was abandoned.
return nullptr;
}
- SkASSERT(this->width() <= fContext->priv().caps()->maxTextureSize() &&
- this->height() <= fContext->priv().caps()->maxTextureSize());
+ SkASSERT(this->width() <= this->context()->priv().caps()->maxTextureSize() &&
+ this->height() <= this->context()->priv().caps()->maxTextureSize());
bool needsCopyForMipsOnly = false;
if (!params.isRepeated() ||
- !GrGpu::IsACopyNeededForRepeatWrapMode(fContext->priv().caps(), proxy.get(),
+ !GrGpu::IsACopyNeededForRepeatWrapMode(this->context()->priv().caps(), proxy.get(),
proxy->width(), proxy->height(), params.filter(),
©Params, scaleAdjust)) {
- needsCopyForMipsOnly = GrGpu::IsACopyNeededForMips(fContext->priv().caps(),
+ needsCopyForMipsOnly = GrGpu::IsACopyNeededForMips(this->context()->priv().caps(),
proxy.get(), params.filter(),
©Params);
if (!needsCopyForMipsOnly) {
diff --git a/src/gpu/GrTextureAdjuster.h b/src/gpu/GrTextureAdjuster.h
index ec008e7..89bf4a2 100644
--- a/src/gpu/GrTextureAdjuster.h
+++ b/src/gpu/GrTextureAdjuster.h
@@ -12,6 +12,8 @@
#include "GrTextureProxy.h"
#include "SkTLazy.h"
+class GrRecordingContext;
+
/**
* Base class for sources that start out as textures. Optionally allows for a content area subrect.
* The intent is not to use content area for subrect rendering. Rather, the pixels outside the
@@ -29,7 +31,7 @@
// We do not ref the texture nor the colorspace, so the caller must keep them in scope while
// this Adjuster is alive.
- GrTextureAdjuster(GrContext*, sk_sp<GrTextureProxy>, SkAlphaType, uint32_t uniqueID,
+ GrTextureAdjuster(GrRecordingContext*, sk_sp<GrTextureProxy>, SkAlphaType, uint32_t uniqueID,
SkColorSpace*);
protected:
diff --git a/src/gpu/GrTextureMaker.cpp b/src/gpu/GrTextureMaker.cpp
index bf71a70..2bdc975 100644
--- a/src/gpu/GrTextureMaker.cpp
+++ b/src/gpu/GrTextureMaker.cpp
@@ -8,16 +8,16 @@
#include "GrTextureMaker.h"
#include "GrColorSpaceXform.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrGpu.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
sk_sp<GrTextureProxy> GrTextureMaker::onRefTextureProxyForParams(const GrSamplerState& params,
bool willBeMipped,
SkScalar scaleAdjust[2]) {
- if (this->width() > fContext->priv().caps()->maxTextureSize() ||
- this->height() > fContext->priv().caps()->maxTextureSize()) {
+ if (this->width() > this->context()->priv().caps()->maxTextureSize() ||
+ this->height() > this->context()->priv().caps()->maxTextureSize()) {
return nullptr;
}
@@ -28,10 +28,10 @@
bool needsCopyForMipsOnly = false;
if (original) {
if (!params.isRepeated() ||
- !GrGpu::IsACopyNeededForRepeatWrapMode(fContext->priv().caps(), original.get(),
+ !GrGpu::IsACopyNeededForRepeatWrapMode(this->context()->priv().caps(), original.get(),
original->width(), original->height(),
params.filter(), ©Params, scaleAdjust)) {
- needsCopyForMipsOnly = GrGpu::IsACopyNeededForMips(fContext->priv().caps(),
+ needsCopyForMipsOnly = GrGpu::IsACopyNeededForMips(this->context()->priv().caps(),
original.get(), params.filter(),
©Params);
if (!needsCopyForMipsOnly) {
@@ -40,14 +40,14 @@
}
} else {
if (!params.isRepeated() ||
- !GrGpu::IsACopyNeededForRepeatWrapMode(fContext->priv().caps(), nullptr,
+ !GrGpu::IsACopyNeededForRepeatWrapMode(this->context()->priv().caps(), nullptr,
this->width(), this->height(),
params.filter(), ©Params, scaleAdjust)) {
return this->refOriginalTextureProxy(willBeMipped, AllowedTexGenType::kAny);
}
}
- GrProxyProvider* proxyProvider = fContext->priv().proxyProvider();
+ GrProxyProvider* proxyProvider = this->context()->priv().proxyProvider();
GrSurfaceOrigin origOrigin = original ? original->origin() : kTopLeft_GrSurfaceOrigin;
GrUniqueKey copyKey;
@@ -74,7 +74,7 @@
return nullptr;
}
- sk_sp<GrTextureProxy> result = CopyOnGpu(fContext, source, copyParams, willBeMipped);
+ sk_sp<GrTextureProxy> result = CopyOnGpu(this->context(), source, copyParams, willBeMipped);
if (!result) {
// If we were unable to make a copy and we only needed a copy for mips, then we will return
diff --git a/src/gpu/GrTextureMaker.h b/src/gpu/GrTextureMaker.h
index e79b976..725a532 100644
--- a/src/gpu/GrTextureMaker.h
+++ b/src/gpu/GrTextureMaker.h
@@ -26,7 +26,7 @@
const GrSamplerState::Filter* filterOrNullForBicubic) override;
protected:
- GrTextureMaker(GrContext* context, int width, int height, bool isAlphaOnly)
+ GrTextureMaker(GrRecordingContext* context, int width, int height, bool isAlphaOnly)
: INHERITED(context, width, height, isAlphaOnly) {}
/**
@@ -39,8 +39,6 @@
virtual sk_sp<GrTextureProxy> refOriginalTextureProxy(bool willBeMipped,
AllowedTexGenType genType) = 0;
- GrContext* context() const { return fContext; }
-
private:
sk_sp<GrTextureProxy> onRefTextureProxyForParams(const GrSamplerState&,
bool willBeMipped,
diff --git a/src/gpu/GrTextureProducer.cpp b/src/gpu/GrTextureProducer.cpp
index fe73bcf..9fe9079 100644
--- a/src/gpu/GrTextureProducer.cpp
+++ b/src/gpu/GrTextureProducer.cpp
@@ -9,6 +9,8 @@
#include "GrClip.h"
#include "GrContextPriv.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrRenderTargetContext.h"
#include "GrTextureProxy.h"
#include "SkGr.h"
@@ -18,7 +20,7 @@
#include "effects/GrSimpleTextureEffect.h"
#include "effects/GrTextureDomain.h"
-sk_sp<GrTextureProxy> GrTextureProducer::CopyOnGpu(GrContext* context,
+sk_sp<GrTextureProxy> GrTextureProducer::CopyOnGpu(GrRecordingContext* context,
sk_sp<GrTextureProxy> inputProxy,
const CopyParams& copyParams,
bool dstWillRequireMipMaps) {
@@ -236,14 +238,14 @@
int mipCount = SkMipMap::ComputeLevelCount(this->width(), this->height());
bool willBeMipped = GrSamplerState::Filter::kMipMap == sampler.filter() && mipCount &&
- fContext->priv().caps()->mipMapSupport();
+ this->context()->priv().caps()->mipMapSupport();
auto result = this->onRefTextureProxyForParams(sampler, willBeMipped, scaleAdjust);
// Check to make sure that if we say the texture willBeMipped that the returned texture has mip
// maps, unless the config is not copyable.
SkASSERT(!result || !willBeMipped || result->mipMapped() == GrMipMapped::kYes ||
- !fContext->priv().caps()->isConfigCopyable(result->config()));
+ !this->context()->priv().caps()->isConfigCopyable(result->config()));
// Check that the "no scaling expected" case always returns a proxy of the same size as the
// producer.
@@ -260,14 +262,14 @@
int mipCount = SkMipMap::ComputeLevelCount(this->width(), this->height());
bool willBeMipped = GrSamplerState::Filter::kMipMap == sampler.filter() && mipCount &&
- fContext->priv().caps()->mipMapSupport();
+ this->context()->priv().caps()->mipMapSupport();
auto result = this->onRefTextureProxyForParams(sampler, willBeMipped, nullptr);
// Check to make sure that if we say the texture willBeMipped that the returned texture has mip
// maps, unless the config is not copyable.
SkASSERT(!result || !willBeMipped || result->mipMapped() == GrMipMapped::kYes ||
- !fContext->priv().caps()->isConfigCopyable(result->config()));
+ !this->context()->priv().caps()->isConfigCopyable(result->config()));
// Check that no scaling occured and we returned a proxy of the same size as the producer.
SkASSERT(!result || (result->width() == this->width() && result->height() == this->height()));
diff --git a/src/gpu/GrTextureProducer.h b/src/gpu/GrTextureProducer.h
index 1994f8b..eb90336 100644
--- a/src/gpu/GrTextureProducer.h
+++ b/src/gpu/GrTextureProducer.h
@@ -13,8 +13,8 @@
#include "SkImageInfo.h"
#include "SkNoncopyable.h"
-class GrContext;
class GrFragmentProcessor;
+class GrRecordingContext;
class GrTexture;
class GrTextureProxy;
class SkColorSpace;
@@ -113,7 +113,7 @@
protected:
friend class GrTextureProducer_TestAccess;
- GrTextureProducer(GrContext* context, int width, int height, bool isAlphaOnly)
+ GrTextureProducer(GrRecordingContext* context, int width, int height, bool isAlphaOnly)
: fContext(context)
, fWidth(width)
, fHeight(height)
@@ -156,7 +156,8 @@
kTightCopy_DomainMode
};
- static sk_sp<GrTextureProxy> CopyOnGpu(GrContext*, sk_sp<GrTextureProxy> inputProxy,
+ // This can draw to accomplish the copy, thus the recording context is needed
+ static sk_sp<GrTextureProxy> CopyOnGpu(GrRecordingContext*, sk_sp<GrTextureProxy> inputProxy,
const CopyParams& copyParams,
bool dstWillRequireMipMaps);
@@ -174,16 +175,17 @@
const SkRect& domain,
const GrSamplerState::Filter* filterOrNullForBicubic);
- GrContext* fContext;
+ GrRecordingContext* context() const { return fContext; }
private:
virtual sk_sp<GrTextureProxy> onRefTextureProxyForParams(const GrSamplerState&,
bool willBeMipped,
SkScalar scaleAdjust[2]) = 0;
- const int fWidth;
- const int fHeight;
- const bool fIsAlphaOnly;
+ GrRecordingContext* fContext;
+ const int fWidth;
+ const int fHeight;
+ const bool fIsAlphaOnly;
typedef SkNoncopyable INHERITED;
};
diff --git a/src/gpu/GrYUVProvider.cpp b/src/gpu/GrYUVProvider.cpp
index a4cab0a..fdbcb79 100644
--- a/src/gpu/GrYUVProvider.cpp
+++ b/src/gpu/GrYUVProvider.cpp
@@ -8,9 +8,9 @@
#include "GrYUVProvider.h"
#include "GrClip.h"
#include "GrColorSpaceXform.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrRenderTargetContext.h"
#include "GrTextureProxy.h"
#include "SkAutoMalloc.h"
@@ -101,7 +101,7 @@
cachedData->unref();
}
-sk_sp<GrTextureProxy> GrYUVProvider::refAsTextureProxy(GrContext* ctx,
+sk_sp<GrTextureProxy> GrYUVProvider::refAsTextureProxy(GrRecordingContext* ctx,
const GrBackendFormat& format,
const GrSurfaceDesc& desc,
SkColorSpace* srcColorSpace,
diff --git a/src/gpu/GrYUVProvider.h b/src/gpu/GrYUVProvider.h
index 633ccfb..e09c37a 100644
--- a/src/gpu/GrYUVProvider.h
+++ b/src/gpu/GrYUVProvider.h
@@ -13,8 +13,8 @@
#include "SkYUVAIndex.h"
#include "SkYUVASizeInfo.h"
-class GrContext;
class GrBackendFormat;
+class GrRecordingContext;
struct GrSurfaceDesc;
class GrTexture;
class GrTextureProxy;
@@ -41,7 +41,7 @@
*
* On failure (e.g. the provider had no data), this returns NULL.
*/
- sk_sp<GrTextureProxy> refAsTextureProxy(GrContext*,
+ sk_sp<GrTextureProxy> refAsTextureProxy(GrRecordingContext*,
const GrBackendFormat&,
const GrSurfaceDesc&,
SkColorSpace* srcColorSpace,
diff --git a/src/gpu/SkGr.cpp b/src/gpu/SkGr.cpp
index 5994657..c9cfadb 100644
--- a/src/gpu/SkGr.cpp
+++ b/src/gpu/SkGr.cpp
@@ -14,6 +14,8 @@
#include "GrGpuResourcePriv.h"
#include "GrPaint.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrTextureProxy.h"
#include "GrTypes.h"
#include "GrXferProcessor.h"
@@ -145,7 +147,8 @@
pixelRef->addGenIDChangeListener(new Invalidator(key, contextUniqueID));
}
-sk_sp<GrTextureProxy> GrCopyBaseMipMapToTextureProxy(GrContext* ctx, GrTextureProxy* baseProxy) {
+sk_sp<GrTextureProxy> GrCopyBaseMipMapToTextureProxy(GrRecordingContext* ctx,
+ GrTextureProxy* baseProxy) {
SkASSERT(baseProxy);
if (!ctx->priv().caps()->isConfigCopyable(baseProxy->config())) {
@@ -179,7 +182,7 @@
return proxy;
}
-sk_sp<GrTextureProxy> GrRefCachedBitmapTextureProxy(GrContext* ctx,
+sk_sp<GrTextureProxy> GrRefCachedBitmapTextureProxy(GrRecordingContext* ctx,
const SkBitmap& bitmap,
const GrSamplerState& params,
SkScalar scaleAdjust[2]) {
@@ -369,7 +372,7 @@
}
#endif
-static inline bool skpaint_to_grpaint_impl(GrContext* context,
+static inline bool skpaint_to_grpaint_impl(GrRecordingContext* context,
const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& skPaint,
const SkMatrix& viewM,
@@ -513,14 +516,14 @@
return true;
}
-bool SkPaintToGrPaint(GrContext* context, const GrColorSpaceInfo& colorSpaceInfo,
+bool SkPaintToGrPaint(GrRecordingContext* context, const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& skPaint, const SkMatrix& viewM, GrPaint* grPaint) {
return skpaint_to_grpaint_impl(context, colorSpaceInfo, skPaint, viewM, nullptr, nullptr,
grPaint);
}
/** Replaces the SkShader (if any) on skPaint with the passed in GrFragmentProcessor. */
-bool SkPaintToGrPaintReplaceShader(GrContext* context,
+bool SkPaintToGrPaintReplaceShader(GrRecordingContext* context,
const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& skPaint,
std::unique_ptr<GrFragmentProcessor> shaderFP,
@@ -533,7 +536,7 @@
}
/** Ignores the SkShader (if any) on skPaint. */
-bool SkPaintToGrPaintNoShader(GrContext* context,
+bool SkPaintToGrPaintNoShader(GrRecordingContext* context,
const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& skPaint,
GrPaint* grPaint) {
@@ -545,7 +548,7 @@
/** Blends the SkPaint's shader (or color if no shader) with a per-primitive color which must
be setup as a vertex attribute using the specified SkBlendMode. */
-bool SkPaintToGrPaintWithXfermode(GrContext* context,
+bool SkPaintToGrPaintWithXfermode(GrRecordingContext* context,
const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& skPaint,
const SkMatrix& viewM,
@@ -555,7 +558,7 @@
grPaint);
}
-bool SkPaintToGrPaintWithTexture(GrContext* context,
+bool SkPaintToGrPaintWithTexture(GrRecordingContext* context,
const GrColorSpaceInfo& colorSpaceInfo,
const SkPaint& paint,
const SkMatrix& viewM,
diff --git a/src/gpu/gradients/GrGradientShader.cpp b/src/gpu/gradients/GrGradientShader.cpp
index 9f3916f..43cf8ae 100644
--- a/src/gpu/gradients/GrGradientShader.cpp
+++ b/src/gpu/gradients/GrGradientShader.cpp
@@ -24,8 +24,6 @@
#include "GrCaps.h"
#include "GrColor.h"
#include "GrColorSpaceInfo.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrRecordingContext.h"
#include "GrRecordingContextPriv.h"
#include "SkGr.h"
diff --git a/src/image/SkImage_Base.h b/src/image/SkImage_Base.h
index b447bf0..c2ac842 100644
--- a/src/image/SkImage_Base.h
+++ b/src/image/SkImage_Base.h
@@ -16,6 +16,7 @@
#include "GrTextureProxy.h"
#include "SkTDArray.h"
+class GrRecordingContext;
class GrTexture;
#endif
@@ -54,7 +55,7 @@
#if SK_SUPPORT_GPU
virtual GrTextureProxy* peekProxy() const { return nullptr; }
virtual sk_sp<GrTextureProxy> asTextureProxyRef() const { return nullptr; }
- virtual sk_sp<GrTextureProxy> asTextureProxyRef(GrContext*, const GrSamplerState&,
+ virtual sk_sp<GrTextureProxy> asTextureProxyRef(GrRecordingContext*, const GrSamplerState&,
SkScalar scaleAdjust[2]) const = 0;
virtual sk_sp<GrTextureProxy> refPinnedTextureProxy(uint32_t* uniqueID) const {
return nullptr;
diff --git a/src/image/SkImage_GpuBase.cpp b/src/image/SkImage_GpuBase.cpp
index fcf0887..a0a8fa6 100644
--- a/src/image/SkImage_GpuBase.cpp
+++ b/src/image/SkImage_GpuBase.cpp
@@ -204,7 +204,7 @@
return true;
}
-sk_sp<GrTextureProxy> SkImage_GpuBase::asTextureProxyRef(GrContext* context,
+sk_sp<GrTextureProxy> SkImage_GpuBase::asTextureProxyRef(GrRecordingContext* context,
const GrSamplerState& params,
SkScalar scaleAdjust[2]) const {
if (!fContext->priv().matches(context)) {
diff --git a/src/image/SkImage_GpuBase.h b/src/image/SkImage_GpuBase.h
index 94efe77..5c5db0e 100644
--- a/src/image/SkImage_GpuBase.h
+++ b/src/image/SkImage_GpuBase.h
@@ -37,7 +37,7 @@
SkASSERT(false);
return this->INHERITED::asTextureProxyRef();
}
- sk_sp<GrTextureProxy> asTextureProxyRef(GrContext*, const GrSamplerState&,
+ sk_sp<GrTextureProxy> asTextureProxyRef(GrRecordingContext*, const GrSamplerState&,
SkScalar scaleAdjust[2]) const final;
sk_sp<GrTextureProxy> refPinnedTextureProxy(uint32_t* uniqueID) const final {
diff --git a/src/image/SkImage_Lazy.cpp b/src/image/SkImage_Lazy.cpp
index 47bf690..db85e95 100644
--- a/src/image/SkImage_Lazy.cpp
+++ b/src/image/SkImage_Lazy.cpp
@@ -16,12 +16,13 @@
#include "SkNextID.h"
#if SK_SUPPORT_GPU
-#include "GrContext.h"
-#include "GrContextPriv.h"
+#include "GrCaps.h"
#include "GrGpuResourcePriv.h"
#include "GrImageTextureMaker.h"
#include "GrResourceKey.h"
#include "GrProxyProvider.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrSamplerState.h"
#include "GrYUVProvider.h"
#include "SkGr.h"
@@ -237,7 +238,7 @@
///////////////////////////////////////////////////////////////////////////////////////////////////
#if SK_SUPPORT_GPU
-sk_sp<GrTextureProxy> SkImage_Lazy::asTextureProxyRef(GrContext* context,
+sk_sp<GrTextureProxy> SkImage_Lazy::asTextureProxyRef(GrRecordingContext* context,
const GrSamplerState& params,
SkScalar scaleAdjust[2]) const {
if (!context) {
@@ -363,7 +364,7 @@
* 4. Ask the generator to return RGB(A) data, which the GPU can convert
*/
sk_sp<GrTextureProxy> SkImage_Lazy::lockTextureProxy(
- GrContext* ctx,
+ GrRecordingContext* ctx,
const GrUniqueKey& origKey,
SkImage::CachingHint chint,
bool willBeMipped,
diff --git a/src/image/SkImage_Lazy.h b/src/image/SkImage_Lazy.h
index 51b893a..e1407e9 100644
--- a/src/image/SkImage_Lazy.h
+++ b/src/image/SkImage_Lazy.h
@@ -46,7 +46,7 @@
bool onReadPixels(const SkImageInfo&, void*, size_t, int srcX, int srcY,
CachingHint) const override;
#if SK_SUPPORT_GPU
- sk_sp<GrTextureProxy> asTextureProxyRef(GrContext*,
+ sk_sp<GrTextureProxy> asTextureProxyRef(GrRecordingContext*,
const GrSamplerState&,
SkScalar scaleAdjust[2]) const override;
sk_sp<SkCachedData> getPlanes(SkYUVASizeInfo*, SkYUVAIndex[4],
@@ -64,7 +64,7 @@
// Returns the texture proxy. If we're going to generate and cache the texture, we should use
// the passed in key (if the key is valid). If genType is AllowedTexGenType::kCheap and the
// texture is not trivial to construct, returns nullptr.
- sk_sp<GrTextureProxy> lockTextureProxy(GrContext*,
+ sk_sp<GrTextureProxy> lockTextureProxy(GrRecordingContext*,
const GrUniqueKey& key,
SkImage::CachingHint,
bool willBeMipped,
diff --git a/src/image/SkImage_Raster.cpp b/src/image/SkImage_Raster.cpp
index 806f366..b1e5e08 100644
--- a/src/image/SkImage_Raster.cpp
+++ b/src/image/SkImage_Raster.cpp
@@ -81,7 +81,7 @@
const SkBitmap* onPeekBitmap() const override { return &fBitmap; }
#if SK_SUPPORT_GPU
- sk_sp<GrTextureProxy> asTextureProxyRef(GrContext*, const GrSamplerState&,
+ sk_sp<GrTextureProxy> asTextureProxyRef(GrRecordingContext*, const GrSamplerState&,
SkScalar scaleAdjust[2]) const override;
#endif
@@ -169,7 +169,7 @@
}
#if SK_SUPPORT_GPU
-sk_sp<GrTextureProxy> SkImage_Raster::asTextureProxyRef(GrContext* context,
+sk_sp<GrTextureProxy> SkImage_Raster::asTextureProxyRef(GrRecordingContext* context,
const GrSamplerState& params,
SkScalar scaleAdjust[2]) const {
if (!context) {
diff --git a/src/image/SkSurface.cpp b/src/image/SkSurface.cpp
index 272e18b..60eb2ad 100644
--- a/src/image/SkSurface.cpp
+++ b/src/image/SkSurface.cpp
@@ -300,7 +300,7 @@
return nullptr;
}
-sk_sp<SkSurface> SkSurface::MakeRenderTarget(GrContext*, const SkSurfaceCharacterization&,
+sk_sp<SkSurface> SkSurface::MakeRenderTarget(GrRecordingContext*, const SkSurfaceCharacterization&,
SkBudgeted) {
return nullptr;
}
diff --git a/src/image/SkSurface_Gpu.cpp b/src/image/SkSurface_Gpu.cpp
index 5865a2e..f5af07f 100644
--- a/src/image/SkSurface_Gpu.cpp
+++ b/src/image/SkSurface_Gpu.cpp
@@ -11,6 +11,8 @@
#include "GrCaps.h"
#include "GrContextPriv.h"
#include "GrContextThreadSafeProxyPriv.h"
+#include "GrRecordingContext.h"
+#include "GrRecordingContextPriv.h"
#include "GrRenderTarget.h"
#include "GrRenderTargetContextPriv.h"
#include "GrRenderTargetProxyPriv.h"
@@ -284,7 +286,7 @@
}
}
-sk_sp<SkSurface> SkSurface::MakeRenderTarget(GrContext* context,
+sk_sp<SkSurface> SkSurface::MakeRenderTarget(GrRecordingContext* context,
const SkSurfaceCharacterization& c,
SkBudgeted budgeted) {
if (!context || !c.isValid()) {
@@ -322,7 +324,9 @@
return nullptr;
}
- sk_sp<SkGpuDevice> device(SkGpuDevice::Make(context, sk_ref_sp(sc->asRenderTargetContext()),
+ // CONTEXT TODO: remove this use of 'backdoor' to create an SkGpuDevice
+ sk_sp<SkGpuDevice> device(SkGpuDevice::Make(context->priv().backdoor(),
+ sk_ref_sp(sc->asRenderTargetContext()),
c.width(), c.height(),
SkGpuDevice::kClear_InitContents));
if (!device) {
diff --git a/src/shaders/SkImageShader.cpp b/src/shaders/SkImageShader.cpp
index bb4aed2..46f4a75 100644
--- a/src/shaders/SkImageShader.cpp
+++ b/src/shaders/SkImageShader.cpp
@@ -160,8 +160,6 @@
#include "GrCaps.h"
#include "GrColorSpaceInfo.h"
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrRecordingContext.h"
#include "GrRecordingContextPriv.h"
#include "SkGr.h"
diff --git a/src/shaders/SkPerlinNoiseShader.cpp b/src/shaders/SkPerlinNoiseShader.cpp
index e81bc0e..1080c69 100644
--- a/src/shaders/SkPerlinNoiseShader.cpp
+++ b/src/shaders/SkPerlinNoiseShader.cpp
@@ -17,8 +17,6 @@
#include "SkWriteBuffer.h"
#if SK_SUPPORT_GPU
-#include "GrContext.h"
-#include "GrContextPriv.h"
#include "GrCoordTransform.h"
#include "GrRecordingContext.h"
#include "GrRecordingContextPriv.h"
diff --git a/src/gpu/SkGr.h b/src/gpu/SkGr.h
index b2bad56..dd18eaf 100644
--- a/src/gpu/SkGr.h
+++ b/src/gpu/SkGr.h
@@ -27,6 +27,7 @@
class GrContext;
class GrFragmentProcessor;
class GrPaint;
+class GrRecordingContext;
class GrResourceProvider;
class GrTextureProxy;
class GrUniqueKey;
@@ -67,16 +68,16 @@
////////////////////////////////////////////////////////////////////////////////
// Paint conversion
/** Converts an SkPaint to a GrPaint for a given GrContext. The matrix is required in order
to convert the SkShader (if any) on the SkPaint. The primitive itself has no color. */
-bool SkPaintToGrPaint(GrContext*,
+bool SkPaintToGrPaint(GrRecordingContext*,
const GrColorSpaceInfo& dstColorSpaceInfo,
const SkPaint& skPaint,
const SkMatrix& viewM,
GrPaint* grPaint);
/** Same as above but ignores the SkShader (if any) on skPaint. */
-bool SkPaintToGrPaintNoShader(GrContext* context,
+bool SkPaintToGrPaintNoShader(GrRecordingContext*,
const GrColorSpaceInfo& dstColorSpaceInfo,
const SkPaint& skPaint,
GrPaint* grPaint);
@@ -84,7 +85,7 @@
/** Replaces the SkShader (if any) on skPaint with the passed in GrFragmentProcessor. The processor
should expect an unpremul input color and produce a premultiplied output color. There is
no primitive color. */
-bool SkPaintToGrPaintReplaceShader(GrContext*,
+bool SkPaintToGrPaintReplaceShader(GrRecordingContext*,
const GrColorSpaceInfo& dstColorSpaceInfo,
const SkPaint& skPaint,
std::unique_ptr<GrFragmentProcessor> shaderFP,
@@ -92,7 +93,7 @@
/** Blends the SkPaint's shader (or color if no shader) with the color which specified via a
GrOp's GrPrimitiveProcesssor. */
-bool SkPaintToGrPaintWithXfermode(GrContext* context,
+bool SkPaintToGrPaintWithXfermode(GrRecordingContext*,
const GrColorSpaceInfo& dstColorSpaceInfo,
const SkPaint& skPaint,
const SkMatrix& viewM,
@@ -103,18 +104,19 @@
the expectation is that the primitive color will be premultiplied, though it really should be
unpremultiplied so that interpolation is done in unpremul space. The paint's alpha will be
applied to the primitive color after interpolation. */
-inline bool SkPaintToGrPaintWithPrimitiveColor(GrContext* context,
+inline bool SkPaintToGrPaintWithPrimitiveColor(GrRecordingContext* context,
const GrColorSpaceInfo& dstColorSpaceInfo,
- const SkPaint& skPaint, GrPaint* grPaint) {
+ const SkPaint& skPaint,
+ GrPaint* grPaint) {
return SkPaintToGrPaintWithXfermode(context, dstColorSpaceInfo, skPaint, SkMatrix::I(),
SkBlendMode::kDst, grPaint);
}
/** This is used when there may or may not be a shader, and the caller wants to plugin a texture
lookup. If there is a shader, then its output will only be used if the texture is alpha8. */
-bool SkPaintToGrPaintWithTexture(GrContext* context,
+bool SkPaintToGrPaintWithTexture(GrRecordingContext*,
const GrColorSpaceInfo& dstColorSpaceInfo,
- const SkPaint& paint,
+ const SkPaint& skPaint,
const SkMatrix& viewM,
std::unique_ptr<GrFragmentProcessor> fp,
bool textureIsAlphaOnly,
@@ -174,7 +176,7 @@
* performed on the absolute texture coordinates (e.g., if the texture is resized out to
* the next power of two). It can be null if the caller is sure the bitmap won't be resized.
*/
-sk_sp<GrTextureProxy> GrRefCachedBitmapTextureProxy(GrContext*,
+sk_sp<GrTextureProxy> GrRefCachedBitmapTextureProxy(GrRecordingContext*,
const SkBitmap&,
const GrSamplerState&,
SkScalar scaleAdjust[2]);
@@ -189,7 +191,7 @@
/**
* Creates a new texture with mipmap levels and copies the baseProxy into the base layer.
*/
-sk_sp<GrTextureProxy> GrCopyBaseMipMapToTextureProxy(GrContext*,
+sk_sp<GrTextureProxy> GrCopyBaseMipMapToTextureProxy(GrRecordingContext*,
GrTextureProxy* baseProxy);
/*
@@ -67,16 +68,16 @@
////////////////////////////////////////////////////////////////////////////////
// Paint conversion
-/** Converts an SkPaint to a GrPaint for a given GrContext. The matrix is required in order
+/** Converts an SkPaint to a GrPaint for a given GrRecordingContext. The matrix is required in order
to convert the SkShader (if any) on the SkPaint. The primitive itself has no color. */